@@ -34,6 +34,7 @@ module Hledger.Data.Dates (
   getCurrentYear,
   nulldate,
   spanContainsDate,
+  periodContainsDate,
   parsedateM,
   parsedate,
   showDate,
@@ -212,6 +213,11 @@ spanContainsDate (DateSpan Nothing (Just e))  d = d < e
 spanContainsDate (DateSpan (Just b) Nothing)  d = d >= b
 spanContainsDate (DateSpan (Just b) (Just e)) d = d >= b && d < e
 
+-- | Does the period include the given date ?
+-- (Here to avoid import cycle).
+periodContainsDate :: Period -> Day -> Bool
+periodContainsDate p = spanContainsDate (periodAsDateSpan p)
+
 -- | Calculate the intersection of a number of datespans.
 spansIntersect [] = nulldatespan
 spansIntersect [d] = d
@@ -324,6 +330,7 @@ fixSmartDateStrEither' d s = case parsewith smartdateonly (T.toLower s) of
 -- | Convert a SmartDate to an absolute date using the provided reference date.
 --
 -- ==== Examples:
+-- >>> :set -XOverloadedStrings
 -- >>> let t = fixSmartDateStr (parsedate "2008/11/26")
 -- >>> t "0000-01-01"
 -- "0000/01/01"
