@@ -32,6 +32,8 @@ module Hledger.Data.Journal (
   journalAccountNamesUsed,
   -- journalAmountAndPriceCommodities,
   journalAmounts,
+  overJournalAmounts,
+  traverseJournalAmounts,
   -- journalCanonicalCommodities,
   journalDateSpan,
   journalDescriptions,
@@ -62,12 +64,14 @@ module Hledger.Data.Journal (
   tests_Hledger_Data_Journal,
 )
 where
+import Control.Applicative (Const(..))
 import Control.Arrow
 import Control.Monad
 import Control.Monad.Except
 import qualified Control.Monad.Reader as R
 import Control.Monad.ST
 import Data.Array.ST
+import Data.Functor.Identity (Identity(..))
 import qualified Data.HashTable.ST.Cuckoo as HT
 import Data.List
 -- import Data.Map (findWithDefault)
@@ -529,9 +533,8 @@ checkBalanceAssertion p@Posting{ pbalanceassertion = Just ass} amt
                         ])
             (case ptransaction p of
                Nothing -> ":" -- shouldn't happen
-               Just t ->  printf " in \"%s\" (line %d, column %d):\nin transaction:\n%s"
-                          f l c (chomp $ show t) :: String
-                 where GenericSourcePos f l c = tsourcepos t)
+               Just t ->  printf " in %s:\nin transaction:\n%s"
+                          (showGenericSourcePos $ tsourcepos t) (chomp $ show t) :: String)
             (showPostingLine p)
             (showDate $ postingDate p)
             (T.unpack $ paccount p) -- XXX pack
@@ -584,8 +587,8 @@ journalBalanceTransactionsST assrt j createStore storeIn extract =
     flip R.runReaderT (Env bals (storeIn txStore) assrt $
                        Just $ jinferredcommodities j) $ do
       dated <- fmap snd . sortBy (comparing fst) . concat
-             <$> mapM discriminateByDate (jtxns j)
-      mapM checkInferAndRegisterAmounts dated
+             <$> mapM' discriminateByDate (jtxns j)
+      mapM' checkInferAndRegisterAmounts dated
     lift $ extract txStore
   where size = genericLength $ journalPostings j
 
@@ -657,7 +660,7 @@ checkInferAndRegisterAmounts (Right oldTx) = do
   where
     inferFromAssignment :: Posting -> CurrentBalancesModifier s Posting
     inferFromAssignment p = maybe (return p)
-      (fmap (\a -> p { pamount = a }) . setBalance (paccount p))
+      (fmap (\a -> p { pamount = a, porigin = Just $ originalPosting p }) . setBalance (paccount p))
       $ pbalanceassertion p
 
 -- | Adds a posting's amonut to the posting's account balance and
@@ -759,7 +762,7 @@ canonicalStyleFrom ss@(first:_) =
   where
     mgrps = maybe Nothing Just $ headMay $ catMaybes $ map asdigitgroups ss
     -- precision is maximum of all precisions
-    prec = maximum $ map asprecision ss
+    prec = maximumStrict $ map asprecision ss
     mdec  = Just $ headDef '.' $ catMaybes $ map asdecimalpoint ss
     -- precision is that of first amount with a decimal point
     -- (mdec, prec) =
@@ -827,12 +830,28 @@ journalConvertAmountsToCost j@Journal{jtxns=ts} = j{jtxns=map fixtransaction ts}
 -- Amounts in posting prices are not used for canonicalisation.
 --
 journalAmounts :: Journal -> [Amount]
-journalAmounts j =
-  concat
-   [map mpamount $ jmarketprices j
-   ,concatMap flatten $ map pamount $ journalPostings j
-   ]
-  where flatten (Mixed as) = as
+journalAmounts = getConst . traverseJournalAmounts (Const . (:[]))
+
+-- | Maps over all of the amounts in the journal
+overJournalAmounts :: (Amount -> Amount) -> Journal -> Journal
+overJournalAmounts f = runIdentity . traverseJournalAmounts (Identity . f)
+
+-- | Traverses over all ofthe amounts in the journal, in the order
+-- indicated by 'journalAmounts'.
+traverseJournalAmounts
+    :: Applicative f
+    => (Amount -> f Amount)
+    -> Journal -> f Journal
+traverseJournalAmounts f j =
+    recombine <$> (traverse . mpa) f (jmarketprices j)
+              <*> (traverse . tp . traverse . pamt . maa . traverse) f (jtxns j)
+  where
+    recombine mps txns = j { jmarketprices = mps, jtxns = txns }
+    -- a bunch of traversals
+    mpa  g mp = (\amt -> mp { mpamount  = amt }) <$> g (mpamount mp)
+    tp   g t  = (\ps  -> t  { tpostings = ps  }) <$> g (tpostings t)
+    pamt g p  = (\amt -> p  { pamount   = amt }) <$> g (pamount p)
+    maa  g (Mixed as) = Mixed <$> g as
 
 -- | The fully specified date span enclosing the dates (primary or secondary)
 -- of all this journal's transactions and postings, or DateSpan Nothing Nothing
@@ -842,8 +861,8 @@ journalDateSpan secondary j
     | null ts   = DateSpan Nothing Nothing
     | otherwise = DateSpan (Just earliest) (Just $ addDays 1 latest)
     where
-      earliest = minimum dates
-      latest   = maximum dates
+      earliest = minimumStrict dates
+      latest   = maximumStrict dates
       dates    = pdates ++ tdates
       tdates   = map (if secondary then transactionDate2 else tdate) ts
       pdates   = concatMap (catMaybes . map (if secondary then (Just . postingDate2) else pdate) . tpostings) ts
@@ -889,7 +908,7 @@ abspat pat = if isnegativepat pat then drop (length negateprefix) pat else pat
 
 -- tests
 
--- A sample journal for testing, similar to data/sample.journal:
+-- A sample journal for testing, similar to examples/sample.journal:
 --
 -- 2008/01/01 income
 --     assets:bank:checking  $1
