@@ -118,6 +118,17 @@ isLastDayOfMonth y m d =
     12 -> d==31
     _ -> False
 
+-- | Is this period a "standard" period, referencing a particular day, week, month, quarter, or year ?
+-- Periods of other durations, or infinite duration, or not starting on a standard period boundary, are not.
+isStandardPeriod = isStandardPeriod' . simplifyPeriod
+  where
+    isStandardPeriod' (DayPeriod _) = True
+    isStandardPeriod' (WeekPeriod _) = True
+    isStandardPeriod' (MonthPeriod _ _) = True
+    isStandardPeriod' (QuarterPeriod _ _) = True
+    isStandardPeriod' (YearPeriod _) = True
+    isStandardPeriod' _ = False
+
 -- | Render a period as a compact display string suitable for user output.
 --
 -- >>> showPeriod (WeekPeriod (fromGregorian 2016 7 25))
@@ -143,7 +154,8 @@ periodEnd p = me
   where
     DateSpan _ me = periodAsDateSpan p
 
--- | Move a period to the following period of same duration.
+-- | Move a standard period to the following period of same duration.
+-- Non-standard periods are unaffected.
 periodNext :: Period -> Period
 periodNext (DayPeriod b) = DayPeriod (addDays 1 b)
 periodNext (WeekPeriod b) = WeekPeriod (addDays 7 b)
@@ -154,7 +166,8 @@ periodNext (QuarterPeriod y q) = QuarterPeriod y (q+1)
 periodNext (YearPeriod y) = YearPeriod (y+1)
 periodNext p = p
 
--- | Move a period to the preceding period of same duration.
+-- | Move a standard period to the preceding period of same duration.
+-- Non-standard periods are unaffected.
 periodPrevious :: Period -> Period
 periodPrevious (DayPeriod b) = DayPeriod (addDays (-1) b)
 periodPrevious (WeekPeriod b) = WeekPeriod (addDays (-7) b)
@@ -165,7 +178,8 @@ periodPrevious (QuarterPeriod y q) = QuarterPeriod y (q-1)
 periodPrevious (YearPeriod y) = YearPeriod (y-1)
 periodPrevious p = p
 
--- | Move a period to the following period of same duration, staying within enclosing dates.
+-- | Move a standard period to the following period of same duration, staying within enclosing dates.
+-- Non-standard periods are unaffected.
 periodNextIn :: DateSpan -> Period -> Period
 periodNextIn (DateSpan _ (Just e)) p =
   case mb of
@@ -176,7 +190,8 @@ periodNextIn (DateSpan _ (Just e)) p =
     mb = periodStart p'
 periodNextIn _ p = periodNext p
 
--- | Move a period to the preceding period of same duration, staying within enclosing dates.
+-- | Move a standard period to the preceding period of same duration, staying within enclosing dates.
+-- Non-standard periods are unaffected.
 periodPreviousIn :: DateSpan -> Period -> Period
 periodPreviousIn (DateSpan (Just b) _) p =
   case me of
@@ -187,11 +202,24 @@ periodPreviousIn (DateSpan (Just b) _) p =
     me = periodEnd p'
 periodPreviousIn _ p = periodPrevious p
 
+-- | Move a standard period stepwise so that it encloses the given date.
+-- Non-standard periods are unaffected.
+periodMoveTo :: Day -> Period -> Period
+periodMoveTo d (DayPeriod _) = DayPeriod d
+periodMoveTo d (WeekPeriod _) = WeekPeriod $ mondayBefore d
+periodMoveTo d (MonthPeriod _ _) = MonthPeriod y m where (y,m,_) = toGregorian d
+periodMoveTo d (QuarterPeriod _ _) = QuarterPeriod y q
+  where
+    (y,m,_) = toGregorian d
+    q = quarterContainingMonth m
+periodMoveTo d (YearPeriod _) = YearPeriod y where (y,_,_) = toGregorian d
+periodMoveTo _ p = p
+
 -- | Enlarge a standard period to the next larger enclosing standard period, if there is one.
 -- Eg, a day becomes the enclosing week.
 -- A week becomes whichever month the week's thursday falls into.
 -- A year becomes all (unlimited).
--- Growing an unlimited period, or a non-standard period (arbitrary dates) has no effect.
+-- Non-standard periods (arbitrary dates, or open-ended) are unaffected.
 periodGrow :: Period -> Period
 periodGrow (DayPeriod b) = WeekPeriod $ mondayBefore b
 periodGrow (WeekPeriod b) = MonthPeriod y m
