@@ -8,7 +8,7 @@ tags.
 
 -}
 
-{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE OverloadedStrings, LambdaCase #-}
 
 module Hledger.Data.Transaction (
   -- * Transaction
@@ -37,6 +37,11 @@ module Hledger.Data.Transaction (
   showTransactionUnelided,
   showTransactionUnelidedOneLineAmounts,
   showPostingLine,
+  showPostingLines,
+  -- * GenericSourcePos
+  sourceFilePath,
+  sourceFirstLine,
+  showGenericSourcePos,
   -- * misc.
   tests_Hledger_Data_Transaction
 )
@@ -66,6 +71,21 @@ instance Show ModifierTransaction where
 instance Show PeriodicTransaction where
     show t = "~ " ++ T.unpack (ptperiodicexpr t) ++ "\n" ++ unlines (map show (ptpostings t))
 
+sourceFilePath :: GenericSourcePos -> FilePath
+sourceFilePath = \case
+    GenericSourcePos fp _ _ -> fp
+    JournalSourcePos fp _ -> fp
+
+sourceFirstLine :: GenericSourcePos -> Int
+sourceFirstLine = \case
+    GenericSourcePos _ line _ -> line
+    JournalSourcePos _ (line, _) -> line
+
+showGenericSourcePos :: GenericSourcePos -> String
+showGenericSourcePos = \case
+    GenericSourcePos fp line column -> show fp ++ " (line " ++ show line ++ ", column " ++ show column ++ ")"
+    JournalSourcePos fp (line, line') -> show fp ++ " (lines " ++ show line ++ "-" ++ show line' ++ ")"
+
 nullsourcepos :: GenericSourcePos
 nullsourcepos = GenericSourcePos "" 1 1
 
@@ -132,7 +152,8 @@ tests_showTransactionUnelided = [
       `gives` unlines [
       "2012/05/14=2012/05/15 (code) desc    ; tcomment1",
       "    ; tcomment2",
-      "                $1.00",
+      "    * a         $1.00",
+      "    ; pcomment2",
       "    * a         2.00h",
       "    ; pcomment2",
       ""
@@ -186,12 +207,13 @@ postingsAsLines elide onelineamounts t ps
     | otherwise = concatMap (postingAsLines False onelineamounts ps) ps
 
 postingAsLines :: Bool -> Bool -> [Posting] -> Posting -> [String]
-postingAsLines elideamount onelineamounts ps p =
-    postinglines
+postingAsLines elideamount onelineamounts ps p = concat [
+    postingblock
     ++ newlinecomments
+    | postingblock <- postingblocks]
   where
-    postinglines = map rstrip $ lines $ concatTopPadded [account, "  ", amount, assertion, samelinecomment]
-    assertion = maybe "" ((" = " ++) . showAmount) $ pbalanceassertion p
+    postingblocks = [map rstrip $ lines $ concatTopPadded [account, "  ", amount, assertion, samelinecomment] | amount <- shownAmounts]
+    assertion = maybe "" ((" = " ++) . showAmountWithZeroCommodity) $ pbalanceassertion p
     account =
       indent $
         showstatus p ++ fitString (Just acctwidth) Nothing False True (showAccountName Nothing (ptype p) (paccount p))
@@ -200,10 +222,10 @@ postingAsLines elideamount onelineamounts ps p =
           acctwidth = maximum $ map (textWidth . paccount) ps
 
     -- currently prices are considered part of the amount string when right-aligning amounts
-    amount
-      | elideamount    = ""
-      | onelineamounts = fitString (Just amtwidth) Nothing False False $ showMixedAmountOneLine $ pamount p
-      | otherwise      = fitStringMulti (Just amtwidth) Nothing False False $ showMixedAmount $ pamount p
+    shownAmounts
+      | elideamount    = [""]
+      | onelineamounts = [fitString (Just amtwidth) Nothing False False $ showMixedAmountOneLine $ pamount p]
+      | otherwise      = map (fitStringMulti (Just amtwidth) Nothing False False . showAmount ) . amounts $ pamount p
       where
         amtwidth = maximum $ 12 : map (strWidth . showMixedAmount . pamount) ps  -- min. 12 for backwards compatibility
 
@@ -220,10 +242,16 @@ showPostingLine p =
   "    " ++
   showMixedAmountOneLine (pamount p)
 
+-- | Produce posting line with all comment lines associated with it
+showPostingLines :: Posting -> [String]
+showPostingLines p = postingAsLines False False ps p where
+    ps | Just t <- ptransaction p = tpostings t
+       | otherwise = [p]
+
 tests_postingAsLines = [
    "postingAsLines" ~: do
-    let p `gives` ls = assertEqual "" ls (postingAsLines False False [p] p)
-    posting `gives` ["                 0"]
+    let p `gives` ls = assertEqual (show p) ls (postingAsLines False False [p] p)
+    posting `gives` []
     posting{
       pstatus=Cleared,
       paccount="a",
@@ -233,13 +261,47 @@ tests_postingAsLines = [
       ptags=[("ptag1","val1"),("ptag2","val2")]
       }
      `gives` [
-      "                $1.00",
+      "    * a         $1.00    ; pcomment1",
+      "    ; pcomment2",
+      "    ;   tag3: val3  ",
       "    * a         2.00h    ; pcomment1",
       "    ; pcomment2",
       "    ;   tag3: val3  "
       ]
  ]
 
+tests_inference = [
+    "inferBalancingAmount" ~: do
+    let p `gives` p' = assertEqual (show p) (Right p') $ inferTransaction p
+        inferTransaction :: Transaction -> Either String Transaction
+        inferTransaction = runIdentity . runExceptT . inferBalancingAmount (\_ _ -> return ())
+    nulltransaction `gives` nulltransaction
+    nulltransaction{
+      tpostings=[
+        "a" `post` usd (-5),
+        "b" `post` missingamt
+      ]}
+      `gives`
+      nulltransaction{
+        tpostings=[
+          "a" `post` usd (-5),
+          "b" `post` usd 5
+        ]}
+    nulltransaction{
+      tpostings=[
+        "a" `post` usd (-5),
+        "b" `post` (eur 3 @@ usd 4),
+        "c" `post` missingamt
+      ]}
+      `gives`
+      nulltransaction{
+        tpostings=[
+          "a" `post` usd (-5),
+          "b" `post` (eur 3 @@ usd 4),
+          "c" `post` usd 1
+        ]}
+ ]
+
 indent :: String -> String
 indent = ("    "++)
 
@@ -357,17 +419,17 @@ inferBalancingAmount update t@Transaction{tpostings=ps}
            return t{tpostings=postings}
   where
     printerr s = intercalate "\n" [s, showTransactionUnelided t]
-    ((amountfulrealps, amountlessrealps), realsum) =
-      (partition hasAmount (realPostings t), sum $ map pamount amountfulrealps)
-    ((amountfulbvps, amountlessbvps), bvsum)       =
-      (partition hasAmount (balancedVirtualPostings t), sum $ map pamount amountfulbvps)
+    (amountfulrealps, amountlessrealps) = partition hasAmount (realPostings t)
+    realsum = sumStrict $ map pamount amountfulrealps
+    (amountfulbvps, amountlessbvps) = partition hasAmount (balancedVirtualPostings t)
+    bvsum = sumStrict $ map pamount amountfulbvps
     inferamount p@Posting{ptype=RegularPosting}
      | not (hasAmount p) = updateAmount p realsum
     inferamount p@Posting{ptype=BalancedVirtualPosting}
      | not (hasAmount p) = updateAmount p bvsum
     inferamount p = return p
-    updateAmount p amt = update (paccount p) amt' >> return p { pamount=amt' }
-      where amt' = costOfMixedAmount (-amt)
+    updateAmount p amt = update (paccount p) amt' >> return p { pamount=amt', porigin=Just $ originalPosting p }
+      where amt' = normaliseMixedAmount $ costOfMixedAmount (-amt)
 
 -- | Infer prices for this transaction's posting amounts, if needed to make
 -- the postings balance, and if possible. This is done once for the real
@@ -424,14 +486,14 @@ priceInferrerFor t pt = inferprice
     pmixedamounts  = map pamount postings
     pamounts       = concatMap amounts pmixedamounts
     pcommodities   = map acommodity pamounts
-    sumamounts     = amounts $ sum pmixedamounts -- sum normalises to one amount per commodity & price
+    sumamounts     = amounts $ sumStrict pmixedamounts -- sum normalises to one amount per commodity & price
     sumcommodities = map acommodity sumamounts
     sumprices      = filter (/=NoPrice) $ map aprice sumamounts
     caninferprices = length sumcommodities == 2 && null sumprices
 
     inferprice p@Posting{pamount=Mixed [a]}
       | caninferprices && ptype p == pt && acommodity a == fromcommodity
-        = p{pamount=Mixed [a{aprice=conversionprice}]}
+        = p{pamount=Mixed [a{aprice=conversionprice}], porigin=Just $ originalPosting p}
       where
         fromcommodity = head $ filter (`elem` sumcommodities) pcommodities -- these heads are ugly but should be safe
         conversionprice
@@ -453,7 +515,8 @@ transactionDate2 t = fromMaybe (tdate t) $ tdate2 t
 -- | Ensure a transaction's postings refer back to it, so that eg
 -- relatedPostings works right.
 txnTieKnot :: Transaction -> Transaction
-txnTieKnot t@Transaction{tpostings=ps} = t{tpostings=map (postingSetTransaction t) ps}
+txnTieKnot t@Transaction{tpostings=ps} = t' where
+    t' = t{tpostings=map (postingSetTransaction t') ps}
 
 -- | Ensure a transaction's postings do not refer back to it, so that eg
 -- recursiveSize and GHCI's :sprint work right.
@@ -467,6 +530,7 @@ postingSetTransaction t p = p{ptransaction=Just t}
 tests_Hledger_Data_Transaction = TestList $ concat [
   tests_postingAsLines,
   tests_showTransactionUnelided,
+  tests_inference,
   [
   "showTransaction" ~: do
      assertEqual "show a balanced transaction, eliding last amount"
