@@ -41,10 +41,8 @@ import Data.Char (isSpace)
 import Data.List (foldl')
 import Data.Maybe
 import Data.Text (Text)
-import qualified Data.Text as T
 import Test.HUnit
 import Text.Megaparsec hiding (parse)
-import System.FilePath
 
 import Hledger.Data
 import Hledger.Read.Common
@@ -53,30 +51,27 @@ import Hledger.Utils hiding (ptrace)
 -- easier to toggle this here sometimes
 -- import qualified Hledger.Utils (ptrace)
 -- ptrace = Hledger.Utils.ptrace
+ptrace :: Monad m => a -> m a
 ptrace = return
 
 reader :: Reader
-reader = Reader format detect parse
-
-format :: String
-format = "timedot"
-
--- | Does the given file path and data look like it might contain this format ?
-detect :: FilePath -> Text -> Bool
-detect f t
-  | f /= "-"  = takeExtension f == '.':format -- from a file: yes if the extension matches the format name
-  | otherwise = regexMatches "(^|\n)[0-9]" $ T.unpack t  -- from stdin: yes if we can see a possible timedot day entry (digits in column 0)
+reader = Reader
+  {rFormat     = "timedot"
+  ,rExtensions = ["timedot"]
+  ,rParser     = parse
+  ,rExperimental = False
+  }
 
 -- | Parse and post-process a "Journal" from the timedot format, or give an error.
 parse :: Maybe FilePath -> Bool -> FilePath -> Text -> ExceptT String IO Journal
 parse _ = parseAndFinaliseJournal timedotfilep
 
-timedotfilep :: ErroringJournalParser ParsedJournal
+timedotfilep :: JournalStateParser m ParsedJournal
 timedotfilep = do many timedotfileitemp
                   eof
                   get
     where
-      timedotfileitemp :: ErroringJournalParser ()
+      timedotfileitemp :: JournalStateParser m ()
       timedotfileitemp = do
         ptrace "timedotfileitemp"
         choice [
@@ -94,7 +89,7 @@ addTransactions ts j = foldl' (flip ($)) j (map addTransaction ts)
 -- biz.research .
 -- inc.client1  .... .... .... .... .... ....
 -- @
-timedotdayp :: ErroringJournalParser [Transaction]
+timedotdayp :: JournalStateParser m [Transaction]
 timedotdayp = do
   ptrace " timedotdayp"
   d <- datep <* lift eolof
@@ -106,7 +101,7 @@ timedotdayp = do
 -- @
 -- fos.haskell  .... ..
 -- @
-timedotentryp :: ErroringJournalParser Transaction
+timedotentryp :: JournalStateParser m Transaction
 timedotentryp = do
   ptrace "  timedotentryp"
   pos <- genericSourcePos <$> getPosition
@@ -130,14 +125,14 @@ timedotentryp = do
         }
   return t
 
-timedotdurationp :: ErroringJournalParser Quantity
+timedotdurationp :: JournalStateParser m Quantity
 timedotdurationp = try timedotnumberp <|> timedotdotsp
 
 -- | Parse a duration written as a decimal number of hours (optionally followed by the letter h).
 -- @
 -- 1.5h
 -- @
-timedotnumberp :: ErroringJournalParser Quantity
+timedotnumberp :: JournalStateParser m Quantity
 timedotnumberp = do
    (q, _, _, _) <- lift numberp
    lift (many spacenonewline)
@@ -149,7 +144,7 @@ timedotnumberp = do
 -- @
 -- .... ..
 -- @
-timedotdotsp :: ErroringJournalParser Quantity
+timedotdotsp :: JournalStateParser m Quantity
 timedotdotsp = do
   dots <- filter (not.isSpace) <$> many (oneOf (". " :: [Char]))
   return $ (/4) $ fromIntegral $ length dots
