@@ -17,7 +17,13 @@ Easy regular expression helpers, currently based on regex-tdfa. These should:
 
 - have simple monomorphic types
 
-- work with strings
+- work with simple strings
+
+Regex strings are automatically compiled into regular expressions the
+first time they are seen, and these are cached. If you use a huge
+number of unique regular expressions this might lead to increased
+memory usage. Several functions have memoised variants (*Memo), which
+also trade space for time.
 
 Current limitations:
 
@@ -34,6 +40,8 @@ module Hledger.Utils.Regex (
   ,regexMatchesCI
   ,regexReplace
   ,regexReplaceCI
+  ,regexReplaceMemo
+  ,regexReplaceCIMemo
   ,regexReplaceBy
   ,regexReplaceByCI
   )
@@ -42,6 +50,7 @@ where
 import Data.Array
 import Data.Char
 import Data.List (foldl')
+import Data.MemoUgly (memo)
 import Text.Regex.TDFA (
   Regex, CompOption(..), ExecOption(..), defaultCompOpt, defaultExecOpt,
   makeRegexOpts, AllMatches(getAllMatches), match, (=~), MatchText
@@ -60,10 +69,10 @@ type Replacement = String
 -- | Convert our string-based regexps to real ones. Can fail if the
 -- string regexp is malformed.
 toRegex :: Regexp -> Regex
-toRegex = makeRegexOpts compOpt execOpt
+toRegex = memo (makeRegexOpts compOpt execOpt)
 
 toRegexCI :: Regexp -> Regex
-toRegexCI = makeRegexOpts compOpt{caseSensitive=False} execOpt
+toRegexCI = memo (makeRegexOpts compOpt{caseSensitive=False} execOpt)
 
 compOpt :: CompOption
 compOpt = defaultCompOpt
@@ -96,6 +105,14 @@ regexReplace re = replaceRegex (toRegex re)
 regexReplaceCI :: Regexp -> Replacement -> String -> String
 regexReplaceCI re = replaceRegex (toRegexCI re)
 
+-- | A memoising version of regexReplace. Caches the result for each
+-- search pattern, replacement pattern, target string tuple.
+regexReplaceMemo :: Regexp -> Replacement -> String -> String
+regexReplaceMemo re repl = memo (regexReplace re repl)
+
+regexReplaceCIMemo :: Regexp -> Replacement -> String -> String
+regexReplaceCIMemo re repl = memo (regexReplaceCI re repl)
+
 --
 
 replaceRegex :: Regex -> Replacement -> String -> String
