@@ -16,6 +16,7 @@ do the right thing, so this file is a no-op and on its way to being removed.
 Not carefully tested.
 
 -}
+-- TODO obsolete ? 
 
 module Hledger.Utils.UTF8IOCompat (
   readFile,
@@ -33,6 +34,7 @@ module Hledger.Utils.UTF8IOCompat (
   toSystemString,
   error',
   userError',
+  usageError,
 )
 where
 
@@ -130,3 +132,8 @@ error' =
 -- | A SystemString-aware version of userError.
 userError' :: String -> IOError
 userError' = userError . toSystemString
+
+-- | A SystemString-aware version of error that adds a usage hint.
+usageError :: String -> a
+usageError = error' . (++ " (use -h to see usage)")  
+
